package
{
	/**
	 * ...
	 * @author Keith Weatherby II
	 * This doesn't even have a proper title but here is the LD 0 "Indirect Interaction"
	 * Catch-up game.  This file is where most of the "magic" happens.
	 * Source code copyright 2011 by Keith Weatherby II
	 * License: Use as you see fit.
	 */
	
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Tilemap;
	import net.flashpunk.World;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Text;
	
	public class GameWorld extends World
	{
		[Embed(source = 'img/tileset.png')] private const TILESET_IMAGE:Class;
		
		public const MAP_W:int = 40;
		public const MAP_H:int = 90;
		public var map:Array;
		public var tileMap:Tilemap;
		public var mapEnt:Entity;
		
		public var player:Player;
		public var state:String;
		public var counter:Number;
		public var time:Number;
		
		public var scoreText:Text;
		public var scoreEnt:Entity;
		
		public var readyText:Text;
		public var readyEnt:Entity;
		
		public var winText:Text;
		public var winEnt:Entity;
		
		public var loseText:Text;
		public var loseEnt:Entity;
		
		public var level:int = 0;
		public var mapH:int = 60;
		
		public var levelText:Text;
		public var levelEnt:Entity;
		
		public var scoreString:String = "Score: ";
		
		public function GameWorld()
		{						
			mapEnt = new Entity(0, 0);

			map = GenerateNewMapArray(MAP_W, MAP_H);
			tileMap = CreateTilemapFromArray(map, TILESET_IMAGE, MAP_W, MAP_H, 8, 8);
			mapEnt.graphic = tileMap;
			mapEnt.width = MAP_W * 8;
			mapEnt.height = MAP_H * 8;
			add(mapEnt);
			
			// cloud entities are practically part of the player
			player = new Player();
			add(player);
			add(player.leftCloud);
			add(player.rightCloud);
			add(player.leftBlow);
			add(player.rightBlow);
			
			state = "level";  // level screen
			
			// for various time related stuffs
			counter = 0;
			time = 3;
			
			Text.size = 16;
			scoreText = new Text("Score: 0000");
			scoreEnt = new Entity(0, 240 - 20, scoreText);
			add(scoreEnt);
			scoreEnt.graphic.scrollX = 0;
			scoreEnt.graphic.scrollY = 0;

			Text.size = 16;
			levelText = new Text("Level: 00");
			levelEnt = new Entity(320 - 80, 240 - 20, levelText);
			add(levelEnt);
			levelEnt.graphic.scrollX = 0;
			levelEnt.graphic.scrollY = 0;
			
			Text.size = 32;
			readyText = new Text("GET READY");
			readyText.color = 0x00FF00;
			readyEnt = new Entity((FP.width - readyText.width) / 2, (FP.height - readyText.height) / 2, readyText);
			add(readyEnt);
			readyEnt.graphic.scrollX = 0;
			readyEnt.graphic.scrollY = 0;
			readyEnt.visible = false;
			
			// originally said "YOU WIN" but since I put in levels you don't really win
			Text.size = 32;
			winText = new Text("SCORE: 0000000");
			winText.color = 0x440088;
			winEnt = new Entity((FP.width - winText.width) / 2, (FP.height - winText.height) / 2, winText);
			add(winEnt);
			winEnt.graphic.scrollX = 0;
			winEnt.graphic.scrollY = 0;
			winEnt.visible = false;
			
			Text.size = 48;
			loseText = new Text("YOU LOSE!!");
			loseText.color = 0xFF0000;
			loseEnt = new Entity((FP.width - loseText.width) / 2, (FP.height - loseText.height) / 2, loseText);
			add(loseEnt);
			loseEnt.graphic.scrollX = 0;
			loseEnt.graphic.scrollY = 0;
			loseEnt.visible = false;
			
			InitGame();
		}
	
		override public function update():void 
		{
			if (state == "level" || state == "ready")
			{
				counter += FP.elapsed;
				if ( counter > time )
				{
					counter = 0;
					if ( state == "level" )
					{
						// use the same level text but move it
						// scale it and color it down on the main screen
						state = "ready";
						levelText.scale = 1;
						levelEnt.x = 320 - 80;
						levelEnt.y = 240 - 20;
						levelText.color = 0xFFFFFF;
						readyEnt.visible = true;
					}
					else
					if ( state == "ready" )
					{
						// we're ready to go
						readyEnt.visible = false;
						state = "move";
						player.state = "move";
					}
				}
			}
			
			if (state == "move")
			{
				// this keeps the camera synced with the player
				if ( player.y > FP.camera.y - 16 )
					FP.camera.y += player.yVel * FP.elapsed;
				if (FP.camera.y > tileMap.height - 240)
					FP.camera.y = tileMap.height - 240;
				
				// greater than the map height in pixels minus 120 means
				// bottom of the map but half way up the screen
				if ( player.y > tileMap.height - 120 )
				{
					// you actually passed the level here
					player.state = "stop";
					state = "win";
					winEnt.visible = true;
					counter = 0;

					winText.text = scoreString;
					winEnt.x = (FP.width - winText.width) / 2;
					winEnt.y = (FP.height - winText.height) / 2;										
				}	
				
				// get the position of the two bottom corners of the raft
				var corner1x:int = player.x;
				var corner2x:int = player.x + player.width;
				var cornery:int = player.y + player.height;
				// find the tiles they cross
				var c1x:int = corner1x / 8;
				var c2x:int = corner2x / 8;
				var cy:int = cornery / 8;
				
				// if we hit rocks stop and show the lose message
				if ((map[cy * MAP_W + c1x] == 3) || (map[cy * MAP_W + c2x] == 3))
				{
					player.state = "stop";
					state = "lose";
					loseEnt.visible = true;
					counter = 0;
				}
				
				scoreString = "Score: " + player.score;
				scoreText.text = scoreString;
			}
			
			// start a new level if you beat the level
			if ( state == "win" )
			{
				counter += FP.elapsed;
				if ( counter > time )
				{
					counter = 0;
					winEnt.visible = false;
					InitLevel();
				}				
			}
			
			// start a new game if you hit a rock
			if ( state == "lose" )
			{
				counter += FP.elapsed;
				if ( counter > time )
				{
					counter = 0;
					loseEnt.visible = false;
					InitGame();
				}				
			}
			
			super.update();
		}
		
		public function InitGame():void
		{
			level = 0;
			mapH = 60;
			player.score = 0;
			player.yAccel = 10;
			InitLevel();
		}
		
		public function InitLevel():void
		{
			// show the new level
			level++;
			var levelString:String = "Level: " + level;
			levelText.text = levelString;
			levelText.color = 0xFFFF00;
			levelText.scale = 2;
			levelEnt.x = (FP.width - (levelText.width * 2)) / 2;
			levelEnt.y = (FP.height - (levelText.height * 2)) / 2;

			// lengthen the height of the map in tiles by a screens worth
			mapH += level * 30;
			
			// randomish rock positions
			map = GenerateNewMapArray(MAP_W, mapH);
			tileMap = CreateTilemapFromArray(map, TILESET_IMAGE, MAP_W, mapH, 8, 8);
			mapEnt.graphic = tileMap;
			mapEnt.width = MAP_W * 8;
			mapEnt.height = mapH * 8;  // pixel height
			state = "level";
			player.InitPlayer();
			counter = 0;
			time = 3;
			readyEnt.visible = false;
			winEnt.visible = false;
			loseEnt.visible = false;
			FP.camera.y = 0;
			player.yAccel += 5;  // speed up 5 pixels more per sec. than the last
		}
		
		public function CreateTilemapFromArray
		(
			aMap:Array, 
			tileset:*, 
			tilesWide:uint, 
			tilesHigh:uint, 
			tileWidth:uint, 
			tileHeight:uint
		):Tilemap
		{
			var tMap:Tilemap = 
				new Tilemap
				(
					tileset, 
					tilesWide * tileWidth, 
					tilesHigh * tileHeight, 
					tileWidth, tileHeight
				);
			
			// add in tiles to make the tilemap
			for (var my:int = 0; my < tilesHigh; my++)
			{
				for (var mx:int = 0; mx < tilesWide; mx++)
				{
					var mapIndex:int = aMap[my * tilesWide + mx];
				
					if(mapIndex > 0)
						tMap.setTile(mx, my, mapIndex);
				}
			}

			return tMap;
		}

		public function GenerateNewMapArray(w:int, h:int):Array
		{
			var mapArray:Array = new Array(w * h);
			
			// init
			for (var i:int = 0; i < w * h; i++)
			{
				mapArray[i] = 0;
			}
			
			// create river
			for (var my:int = 0; my < h; my++)
			{
				var leftbankSize:int = randomNumber(9, 12);
				var rightbankSize:int = randomNumber(9, 12);
				var riverSize:int = w - (leftbankSize + rightbankSize);
				for (var mx:int = 0; mx < w; mx++)
				{
					if (mx < leftbankSize)
					{
						mapArray[my * w + mx] = 1;
					}
					else if (mx >= leftbankSize && mx < leftbankSize + riverSize)
					{
						mapArray[my * w + mx] = 2;
					}
					else
					{
						mapArray[my * w + mx] = 1;
					}
					
				}
				
			}			
			
			// create rocks
			for (my = 15; my < h - 15; my += 8)
			{
				riverSize = w - 24;
				var numRocks:int = randomNumber(0, 2);
				var r1:int, r2:int;
				
				r1 = -1; r2 = -1;
				
				var rockAreaSize:int  = riverSize / numRocks;
				
				if ( numRocks == 1 )
					r1 = randomNumber(12, 12 + rockAreaSize);
				else if (numRocks == 2)
				{
					r1 = randomNumber(13, 13 + rockAreaSize);
					r2 = randomNumber(r1 + 4, w - 13);
				}

				for (mx = 0; mx < w; mx++)
				{
					if (mx >= 12 && mx < 12 + riverSize)
					{
						if ( r1 > -1 )
						{
							if( r1 == mx )
								mapArray[my * w + mx] = 3;
						}
						
						if ( r2 > -1 )
						{
							if( r2 == mx )
								mapArray[my * w + mx] = 3;
						}						
					}
				}
				
			}			

			return(mapArray);
		}
		
		/** 
		* Generates a truly "random" number
		* @return Random Number
		*/ 
		public function randomNumber(low:Number=0, high:Number=1):Number
		{
		  return Math.floor(Math.random() * (1+high-low)) + low;
		}

	}
}